@@ -11,6 +11,7 @@ Checking url, like:
 http://10.32.8.164:8080/DataService?requestType=getLatestDataName&directory=ECMWF_HR/TMP/850&fileName=&filter=*.024
 """
 
+from pickle import FALSE
 import warnings
 import re
 import http.client
@@ -154,7 +155,7 @@ def get_latest_initTime(directory, suffix="*.006"):
 def get_model_grid(directory, filename=None, suffix="*.024",
                    varname='data', varattrs={'units':''}, scale_off=None,
                    levattrs={'long_name':'pressure_level', 'units':'hPa',
-                             '_CoordinateAxisType':'Pressure'}, cache=True):
+                             '_CoordinateAxisType':'Pressure'}, cache=True, cache_clear=True):
     """
     Retrieve numeric model grid forecast from MICAPS cassandra service.
     Support ensemble member forecast.
@@ -197,7 +198,7 @@ def get_model_grid(directory, filename=None, suffix="*.024",
 
     # retrieve data from cached file
     if cache:
-        cache_file = CONFIG.get_cache_file(directory, filename, name="MICAPS_DATA")
+        cache_file = CONFIG.get_cache_file(directory, filename, name="MICAPS_DATA", cache_clear=cache_clear)
         if cache_file.is_file():
             with open(cache_file, 'rb') as f:
                 data = pickle.load(f)
@@ -307,19 +308,18 @@ def get_model_grid(directory, filename=None, suffix="*.024",
             init_time = np.array([init_time], dtype='datetime64[ms]')
             time = np.array([time], dtype='datetime64[ms]')
 
-            # construct ensemble number
-            if nmem != 0:
-                number = np.arange(nmem)
-
             # define coordinates
             time_coord = ('time', time)
             lon_coord = ('lon', lon, {
-                'long_name':'longitude', 'units':'degrees_east', '_CoordinateAxisType':'Lon'})
+                'long_name':'longitude', 'units':'degrees_east',
+                '_CoordinateAxisType':'Lon', "axis": "X"})
             lat_coord = ('lat', lat, {
-                'long_name':'latitude', 'units':'degrees_north', '_CoordinateAxisType':'Lat'})
+                'long_name':'latitude', 'units':'degrees_north',
+                '_CoordinateAxisType':'Lat', 'axis': "Y"})
             if level[0] != 0:
                 level_coord = ('level', level, levattrs)
             if nmem != 0:
+                number = np.arange(nmem)
                 number_coord = ('number', number, {'_CoordinateAxisType':'Ensemble'})
 
             # create to xarray
@@ -359,6 +359,10 @@ def get_model_grid(directory, filename=None, suffix="*.024",
                 if nmem == 0:
                     speed = np.squeeze(data[0, :, :])
                     angle = np.squeeze(data[1, :, :])
+                    # 原始数据文件中存储为: 西风为0度，南风为90度，东风为180度，北风为270度
+                    # 修改为气象风向常规定义: 北方为0度, 东风为90度, 南风为180度, 西方为270度
+                    angle = 270. - angle
+                    angle[angle<0] = angle[angle<0] + 360.
                     if level[0] == 0:
                         speed = speed[np.newaxis, ...]
                         angle = angle[np.newaxis, ...]
@@ -377,6 +381,10 @@ def get_model_grid(directory, filename=None, suffix="*.024",
                 else:
                     speed = np.squeeze(data[0, :, :, :])
                     angle = np.squeeze(data[1, :, :, :])
+                    # 原始数据文件中存储为: 西风为0度，南风为90度，东风为180度，北风为270度
+                    # 修改为气象风向常规定义: 北方为0度, 东风为90度, 南风为180度, 西方为270度
+                    angle = 270. - angle
+                    angle[angle<0] = angle[angle<0] + 360.
                     if level[0] == 0:
                         speed = speed[np.newaxis, ...]
                         angle = angle[np.newaxis, ...]
@@ -593,7 +601,7 @@ def get_model_profiles(directory, filenames, levels, points, **kargs):
         return None
 
 
-def get_station_data(directory, filename=None, suffix="*.000", dropna=True, cache=True):
+def get_station_data(directory, filename=None, suffix="*.000", dropna=True, cache=True, cache_clear=True):
     """
     Retrieve station data from MICAPS cassandra service.
 
@@ -631,7 +639,7 @@ def get_station_data(directory, filename=None, suffix="*.000", dropna=True, cach
 
     # retrieve data from cached file
     if cache:
-        cache_file = CONFIG.get_cache_file(directory, filename, name="MICAPS_DATA")
+        cache_file = CONFIG.get_cache_file(directory, filename, name="MICAPS_DATA", cache_clear=cache_clear)
         if cache_file.is_file():
             with open(cache_file, 'rb') as f:
                 data = pickle.load(f)
@@ -795,7 +803,7 @@ def get_station_dataset(directory, filenames, allExists=True, pbar=False, **karg
     return pd.concat(dataset)
 
 
-def get_fy_awx(directory, filename=None, suffix="*.AWX", units='', cache=True):
+def get_fy_awx(directory, filename=None, suffix="*.AWX", units='', cache=True, cache_clear=True):
     """
     Retrieve FY satellite cloud awx format file.
     The awx file format is refered to “气象卫星分发产品及其格式规范AWX2.1”
@@ -835,7 +843,7 @@ def get_fy_awx(directory, filename=None, suffix="*.AWX", units='', cache=True):
 
     # retrieve data from cached file
     if cache:
-        cache_file = CONFIG.get_cache_file(directory, filename, name="MICAPS_DATA")
+        cache_file = CONFIG.get_cache_file(directory, filename, name="MICAPS_DATA", cache_clear=cache_clear)
         if cache_file.is_file():
             with open(cache_file, 'rb') as f:
                 data = pickle.load(f)
@@ -957,13 +965,20 @@ def get_fy_awx(directory, filename=None, suffix="*.AWX", units='', cache=True):
                     data = calibration_table[data]
                 data.shape = (head1_info['dataRecordNumber'][0], head1_info['recordLength'][0])
 
-                    # construct longitude and latitude coordinates
-                lat = (
-                    head2_info['latitudeOfNorth'][0]/100. - 
-                    np.arange(head2_info['heightOfImage'][0])*head2_info['verticalResolution'][0]/100.)
-                lon = (
-                    head2_info['longitudeOfWest'][0]/100. + 
-                    np.arange(head2_info['widthOfImage'][0])*head2_info['horizontalResolution'][0]/100.)
+                # construct longitude and latitude coordinates
+                # if use the verticalResolution and horizontalResolution, lon and lat will not be correct.
+                #lat = (
+                #    head2_info['latitudeOfNorth'][0]/100. - 
+                #    np.arange(head2_info['heightOfImage'][0])*head2_info['verticalResolution'][0]/100.)
+                #lon = (
+                #    head2_info['longitudeOfWest'][0]/100. + 
+                #    np.arange(head2_info['widthOfImage'][0])*head2_info['horizontalResolution'][0]/100.)
+                lat = np.linspace(
+                    head2_info['latitudeOfSouth'][0]/100., head2_info['latitudeOfNorth'][0]/100.,
+                    num=head2_info['heightOfImage'][0])
+                lon = np.linspace(
+                    head2_info['longitudeOfWest'][0]/100., head2_info['longitudeOfEast'][0]/100.,
+                    num=head2_info['widthOfImage'][0])
                 
                 # construct time
                 time = datetime(
@@ -974,13 +989,16 @@ def get_fy_awx(directory, filename=None, suffix="*.AWX", units='', cache=True):
                 # define coordinates
                 time_coord = ('time', time)
                 lon_coord = ('lon', lon, {
-                    'long_name':'longitude', 'units':'degrees_east', '_CoordinateAxisType':'Lon'})
+                    'long_name':'longitude', 'units':'degrees_east',
+                    '_CoordinateAxisType':'Lon', 'axis': "X"})
                 lat_coord = ('lat', lat, {
-                    'long_name':'latitude', 'units':'degrees_north', '_CoordinateAxisType':'Lat'})
-                channel_coord = ('channel', head2_info['channel'][0], {'long_name':'channel', 'units':''})
+                    'long_name':'latitude', 'units':'degrees_north',
+                    '_CoordinateAxisType':'Lat', 'axis': "Y"})
+                channel_coord = ('channel', [head2_info['channel'][0]],
+                                 {'long_name':'channel', 'units':''})
 
                 # create xarray
-                data = data[np.newaxis, ...]
+                data = data[np.newaxis, np.newaxis, ...]
                 varattrs = {
                     'productCategory': head1_info['productCategory'][0],   # 产品类型, 1:静止, 2:极轨, 3:格点, 4:离散, 5:图形和分析
                     'formatString': head1_info['formatString'][0],         # 产品格式名称
@@ -989,8 +1007,9 @@ def get_fy_awx(directory, filename=None, suffix="*.AWX", units='', cache=True):
                     'flagOfProjection': head2_info['flagOfProjection'][0], # 投影方式, 0:未投影, 1:兰勃托, 2:麦卡托, 3:极射, 4:等经纬, 5:等面积
                     'units': units}
                 data = xr.Dataset({
-                    'image':(['time', 'lat', 'lon'], data, varattrs)},
-                    coords={ 'time':time_coord, 'lat':lat_coord, 'lon':lon_coord})
+                    'image':(['time', 'channel', 'lat', 'lon'], data, varattrs)},
+                    coords={ 'time':time_coord, 'channel':channel_coord,
+                    'lat':lat_coord, 'lon':lon_coord})
 
                 # add attributes
                 data.attrs['Conventions'] = "CF-1.6"
@@ -1041,7 +1060,7 @@ def get_fy_awxs(directory, filenames, allExists=True, pbar=False, **kargs):
     return xr.concat(dataset, dim='time')
 
 
-def get_radar_mosaic(directory, filename=None, suffix="*.LATLON", cache=True):
+def get_radar_mosaic(directory, filename=None, suffix="*.LATLON", cache=True, cache_clear=True):
     """
     该程序主要用于读取和处理中国气象局CRaMS系统的雷达回波全国拼图数据.
 
@@ -1054,6 +1073,10 @@ def get_radar_mosaic(directory, filename=None, suffix="*.LATLON", cache=True):
 
     :Example:
     >>> data = get_radar_mosaic("RADARMOSAIC/CREF/")
+    >>> dir_dir = "RADARMOSAIC/CREF/"
+    >>> filename = "ACHN.CREF000.20201021.000000.LATLON"
+    >>> CREF = get_radar_mosaic(dir_dir, filename=filename, cache=False)
+    >>> print(CREF['time'].values)
     """
 
     # get data file name
@@ -1077,7 +1100,7 @@ def get_radar_mosaic(directory, filename=None, suffix="*.LATLON", cache=True):
 
     # retrieve data from cached file
     if cache:
-        cache_file = CONFIG.get_cache_file(directory, filename, name="MICAPS_DATA")
+        cache_file = CONFIG.get_cache_file(directory, filename, name="MICAPS_DATA", cache_clear=cache_clear)
         if cache_file.is_file():
             with open(cache_file, 'rb') as f:
                 data = pickle.load(f)
@@ -1181,18 +1204,21 @@ def get_radar_mosaic(directory, filename=None, suffix="*.LATLON", cache=True):
             lat = lat[::-1]
 
             # set time coordinates
+            # 直接使用时间有问题, 需要天数减去1, 秒数加上28800(8小时)
             time = datetime(1970, 1, 1) + timedelta(
-                days=head_info['dates'][0].astype(np.float64),
-                seconds=head_info['seconds'][0].astype(np.float64))
+                days=head_info['dates'][0].astype(np.float64)-1,
+                seconds=head_info['seconds'][0].astype(np.float64)+28800)
             time = np.array([time], dtype='datetime64[m]')
             data = np.expand_dims(data, axis=0)
 
             # define coordinates
             time_coord = ('time', time)
             lon_coord = ('lon', lon, {
-                'long_name':'longitude', 'units':'degrees_east', '_CoordinateAxisType':'Lon'})
+                'long_name':'longitude', 'units':'degrees_east',
+                '_CoordinateAxisType':'Lon', "axis": "X"})
             lat_coord = ('lat', lat, {
-                'long_name':'latitude', 'units':'degrees_north', '_CoordinateAxisType':'Lat'})
+                'long_name':'latitude', 'units':'degrees_north',
+                '_CoordinateAxisType':'Lat', "axis": "Y"})
 
             # create xarray
             varattrs = {'long_name': longname.get(varname, 'radar mosaic'), 
@@ -1246,7 +1272,9 @@ def get_radar_mosaics(directory, filenames, allExists=True, pbar=False, **kargs)
     return xr.concat(dataset, dim='time')
 
 
-def get_tlogp(directory, filename=None, suffix="*.000", cache=True):
+def get_tlogp(directory, filename=None, suffix="*.000",
+              remove_duplicate=False, remove_na=False,
+              cache=False, cache_clear=True):
     """
     该程序用于读取micaps服务器上TLOGP数据信息, 文件格式与MICAPS第5类格式相同.
 
@@ -1254,6 +1282,8 @@ def get_tlogp(directory, filename=None, suffix="*.000", cache=True):
     :param filename: the data filename, if none, will be the latest file.
     :param suffix: the filename filter pattern which will be used to
                    find the specified file.
+    :param remove_duplicate: boolean, the duplicate records will be removed.
+    :param remove_na: boolean, the na records will be removed.
     :param cache: cache retrieved data to local directory, default is True.
     :return: pandas DataFrame object.
 
@@ -1281,7 +1311,7 @@ def get_tlogp(directory, filename=None, suffix="*.000", cache=True):
 
     # retrieve data from cached file
     if cache:
-        cache_file = CONFIG.get_cache_file(directory, filename, name="MICAPS_DATA")
+        cache_file = CONFIG.get_cache_file(directory, filename, name="MICAPS_DATA", cache_clear=cache_clear)
         if cache_file.is_file():
             with open(cache_file, 'rb') as f:
                 records = pickle.load(f)
@@ -1344,8 +1374,8 @@ def get_tlogp(directory, filename=None, suffix="*.000", cache=True):
                         'time': time,
                         'p': float(txt[index]), 'h': float(txt[index+1]),
                         't': float(txt[index+2]), 'td': float(txt[index+3]),
-                        'wind_angle': float(txt[index+4]),
-                        'wind_speed': float(txt[index+5])}
+                        'wd': float(txt[index+4]),
+                        'ws': float(txt[index+5])}
                     records.append(record)
                     index += 6
 
@@ -1353,6 +1383,16 @@ def get_tlogp(directory, filename=None, suffix="*.000", cache=True):
             records = pd.DataFrame(records)
             records.set_index('ID')
 
+            # dealing missing values
+            records = records.replace(9999.0, np.nan)
+            if remove_duplicate:
+                records = records.drop_duplicates()
+            if remove_na:
+                records = records.dropna(subset=['p', 'h', 't', 'td'])
+
+            # the sounding height value convert to meters by multiple 10
+            records['h'] = records['h'] * 10.0
+
             # cache data
             if cache:
                 with open(cache_file, 'wb') as f:
@@ -1397,7 +1437,7 @@ def get_tlogps(directory, filenames, allExists=True, pbar=False, **kargs):
 
 def get_swan_radar(directory, filename=None, suffix="*.000", scale=[0.1, 0], 
                    varattrs={'long_name': 'quantitative_precipitation_forecast', 'short_name': 'QPF', 'units': 'mm'},
-                   cache=True, attach_forecast_period=True):
+                   cache=True, cache_clear=True, attach_forecast_period=True):
     """
     该程序用于读取micaps服务器上SWAN的D131格点数据格式.
     refer to https://www.taodocs.com/p-274692126.html
@@ -1437,7 +1477,7 @@ def get_swan_radar(directory, filename=None, suffix="*.000", scale=[0.1, 0],
 
     # retrieve data from cached file
     if cache:
-        cache_file = CONFIG.get_cache_file(directory, filename, name="MICAPS_DATA")
+        cache_file = CONFIG.get_cache_file(directory, filename, name="MICAPS_DATA", cache_clear=cache_clear)
         if cache_file.is_file():
             with open(cache_file, 'rb') as f:
                 data = pickle.load(f)
@@ -1538,9 +1578,11 @@ def get_swan_radar(directory, filename=None, suffix="*.000", scale=[0.1, 0],
             # define coordinates
             time_coord = ('time', time)
             lon_coord = ('lon', lon, {
-                'long_name':'longitude', 'units':'degrees_east', '_CoordinateAxisType':'Lon'})
+                'long_name':'longitude', 'units':'degrees_east',
+                '_CoordinateAxisType':'Lon', "axis": "X"})
             lat_coord = ('lat', lat, {
-                'long_name':'latitude', 'units':'degrees_north', '_CoordinateAxisType':'Lat'})
+                'long_name':'latitude', 'units':'degrees_north',
+                '_CoordinateAxisType':'Lat', "axis": "Y"})
             level_coord = ('level', level, {
                 'long_name':'height', 'units':'m'})
 
@@ -1600,7 +1642,7 @@ def get_swan_radars(directory, filenames, allExists=True, pbar=False, **kargs):
     return xr.concat(dataset, dim='time')
 
 
-def get_radar_standard(directory, filename=None, suffix="*.BZ2", cache=True):
+def get_radar_standard(directory, filename=None, suffix="*.BZ2", cache=True, cache_clear=True):
     """
     该程序用于读取Micaps服务器上的单站雷达基数据, 该数据为
     "天气雷达基数据标准格式(V1.0版)", 返回数据类型为PyCINRAD的标准雷达数据类.
@@ -1644,7 +1686,7 @@ def get_radar_standard(directory, filename=None, suffix="*.BZ2", cache=True):
     # retrieve data from cached file
     byteArray = None
     if cache:
-        cache_file = CONFIG.get_cache_file(directory, filename, name="MICAPS_DATA")
+        cache_file = CONFIG.get_cache_file(directory, filename, name="MICAPS_DATA", cache_clear=cache_clear)
         if cache_file.is_file():
             with open(cache_file, 'rb') as f:
                 byteArray = pickle.load(f) 
